@@ -2,7 +2,7 @@ package PDF::Extract;
 use strict;
 #use warnings;
 use vars qw($VERSION);
-$VERSION = '2.05';
+$VERSION = '2.06';
 
 =head1 NAME
 
@@ -10,6 +10,7 @@ PDF::Extract - Extracting sub PDF documents from a multi page PDF document
 
 =head1 SYNOPSIS
 
+
  use PDF::Extract;
  $pdf=new PDF::Extract;
  $pdf->servePDFExtract( PDFDoc=>"c:/Docs/my.pdf", PDFPages=>"1-3 31-36" );
@@ -38,6 +39,7 @@ as a new PDF document from a pre-existing PDF document.
 
 With PDF::Extract a new PDF document can be:-
 
+
 =over 4
 
 =item * 
@@ -50,6 +52,7 @@ saved to disk with savePDFExtract.
 
 =item * 
 
+
 printed to STDOUT as a PDF web document with servePDFExtract.
 
 =item * 
@@ -104,8 +107,10 @@ my $CRLF = '[ \t\r\n\f\0]'."*(?:\015|\012|(?:\015\012))";
 Creates a new Extract object with empty state information ready for processing
 data both input and output. New can be called with a hash array argument.
 
+
  new PDF::Extract( PDFDoc=>"c:/Docs/my.pdf", PDFPages=>"1-3 31-36" )
 
+
 This will cause a new PDF document to be generated unless there is an error.
 Extract->new() simply calls getPDFExtract() if there is an argument.
 
@@ -163,10 +168,12 @@ Otherwise savePDFExtract will return "1" and the saved PDF location and file nam
 
 
 sub savePDFExtract{
+
 	&setEnv(@_);    
 	&getDoc;
 	&savePdfDoc;
 	$vars{"PDFError"} ? 0 : 1;
+
 }
 
 =head2  servePDFExtract
@@ -186,6 +193,7 @@ Otherwise servePDFExtract will return "1"
 sub servePDFExtract{
     &setEnv(@_);    
     &getDoc;
+
     &uploadPDFDoc;
 	$vars{"PDFError"} ? 0 : 1;
 }
@@ -216,6 +224,7 @@ fastServePDFExtract will return "1" on success.
 sub fastServePDFExtract{
 	&setEnv(@_);    
 	&redirect if -e "$vars{\"PDFCache\"}/$vars{\"PDFFilename\"} ";
+
 	&getDoc;
 	&savePdfDoc;
 	&redirect;
@@ -307,6 +316,7 @@ PDFDoc will be an empty string if there was an error.
 (set and get)
  
  $pages=$pdf->setVars( PDFPages =>"1 18-23");
+
  or
  $pages=$pdf->getVars("PDFPages");
 
@@ -350,7 +360,9 @@ PDFFilename will contain the actual filename used for the last extracted pdf'.
 =head2 PDFErrorPage
 (set and get)
 
+
  $errorPagePath=$pdf->setVars("PDFErrorPage"=>"C:/myError.html");
+
  or
  $errorPagePath=$pdf->getVars("PDFErrorPage");
 
@@ -364,6 +376,7 @@ The default page has a message in red at the top,
 "There is system problem in processing your PDF Pages request.", 
 and then a description of the actual error follows underneath in black.
 
+
 =head2 PDFExtract 
 (get only)
  
@@ -373,6 +386,7 @@ This variable contains the last PDF document processed by getPDFExtract, savePDF
 PDFExtract will be an empty string if there was an error.
 
 =head2 PDFPagesFound
+
 (get only)
  
  $pagesFound=$pdf->getVars("PDFPagesFound");
@@ -394,6 +408,7 @@ PDFPageCount will be an empty string if there was an error in finding any pages.
 =head2 PDFFileName
 (get only)
 
+
  $filename=$pdf->getVars("PDFFilename");
  
 This variable will contain the actual filename.
@@ -402,17 +417,20 @@ requested page numbers and the .pdf file type suffix. If more than one page is e
 the page numbers will be separated with an underscore "_" for individual pages,  ".." for a range of pages.
 eg. my6.pdf for a single page (page 6) and my1_3..6.pdf  for a multi page PDF (pages 1, 3, 4, 5, 6).
 If PDFSaveAs is set then PDFSaveAs will be used to construct PDFFilename.
+
 The full path to the extracted pdf file can be obtained by - 
 
  $fullpath = $pdf->getVars("PDFCache") ."/". $pdf->getVars("PDFFilename");
  or
  ($path,$filename) = $pdf->getVars("PDFCache","PDFFilename");
     
+
 =head2 PDFError
 (get only)
 
  $error=$pdf->getVars("PDFError");
 
+
 This variable contains a string describing the errors if any in processing the original PDF file.
 PDFError is guarenteed to be set if  getPDFExtract, savePDFExtract, servePDFExtract or fastServePDFExtract fail and return a "0".
 PDFError will be an empty string if there was no error.
@@ -421,8 +439,10 @@ PDFError will be an empty string if there was no error.
 (set for method call duration only)
 
  $pdf->setVars(
+
             PDFDoc=>'C:\docs\pdf', 
             PDFPages=>"2 6-8 ",
+
             PDFDebug=>1);
 
 This really a directive and not a true variable. It is used to debug the setting of variables in a PDF::Extract method call.
@@ -432,6 +452,7 @@ PDFDebug as used above will print:-
 	PDFDoc="C:\docs\pdf/"
 	PDFPages="2 6-8 "
 	PDFDebug="1"
+
  These variables have been set
 	PDFCache="C:/myCache"
 	PDFFilename="2_6..8_.pdf"
@@ -447,9 +468,11 @@ PDFDebug as used above will print:-
 
 # ----------------------------------------------------------- The Private Functions --------------------------------------------------------------
 
+
 sub setEnv {
     my (undef,  %PDF)=@_;
     my $requestedPages=0;
+
     $vars{"PDFError"}="";
     if ($PDF{"PDFDebug"} ) {
         print "These variables are to be set\n";
@@ -478,12 +501,15 @@ sub setEnv {
         @object=@obj=@pages=(); 
         %pageObject=();
 
+
         $filename=$1 if  $PDF{"PDFDoc"}=~/([^\\\/]+)\.pdf$/i;
         if ( -f $PDF{"PDFDoc"} ) {
 	        if (open FILE, $PDF{"PDFDoc"} ) {
+
 				binmode FILE;
 				$pdfFile = join('', <FILE>);
 				close FILE;
+
 		        $vars{"PDFDoc"}=$PDF{"PDFDoc"};
 		    } else {
 		        &error( "Can't open PDF document  $PDF{\"PDFDoc\"} to read\n",__FILE__,__LINE__);	    
@@ -499,6 +525,7 @@ sub setEnv {
         @object=@obj=@pages=(); 
         %getPages=%pageObject=();
 
+
 		$pages=$PDF{ "PDFPages" };
 		my $pageError=$pages;
 		$pages=~s/\.\./-/g;
@@ -518,9 +545,11 @@ sub setEnv {
 		    foreach my $page ( sort  keys %getPages) { 
 		        $pages.="$page, ";
 		    }
+
 		    $pages=~s/, $//;
 		    $vars{ "PDFPages"}=$pages;
 		} else {
+
 		    &error("Can't get PDF Pages. No page numbers were set with '$pages' ",__FILE__,__LINE__);
 		}
 	 }
@@ -532,6 +561,7 @@ sub setEnv {
         $vars{"PDFSaveAs"}=~s/\.pdf$//i;    # just want the name, not the path and not the .pdf tag
         $vars{"PDFSaveAs"}=~s/^.*[\/\\]//;
      }
+
     $vars{"PDFFilename"}=$vars{"PDFSaveAs"} ? $vars{"PDFSaveAs"}.".pdf" : "$filename$fileNumber.pdf";
 
     if ( $PDF{"PDFDebug"} ) {
@@ -545,16 +575,19 @@ sub setEnv {
 sub dir {
 	my($path,$dir,@folders)=@_;
 	$path=~s/\\/\//g;
+
 	(@folders)=split "/", $path;
 	foreach my $folder (@folders) {
 		$dir.= $folder=~/:/ ? $folder : "/$folder";
 		next if $folder=~/:/;
+
 		mkdir $dir, 0x777 unless -d $dir;
 		#   print "$dir\n";
 	}
 	$path=~s/\//\\/g if $ENV{"OS"}=~/Windows/i;
 	return &error("This Cache path \"$path\" can't be created",__FILE__,__LINE__) 
 	    unless -d $path;
+
 	$path;
 }
 
@@ -593,6 +626,8 @@ Content-Type: application/pdf\r
 \r
 $vars{"PDFExtract"}\r
 EOF
+
+
 }
 
 #------------------------------------ support  Routines --------------------------------------------
@@ -616,6 +651,7 @@ sub error {
 	"";
 }
 
+
 #------------------------------------ PDF Page Routines --------------------------------------------
 sub getRoot {
 	return "" if $vars{"PDFError"};
@@ -628,17 +664,17 @@ sub getRoot {
     &getObj($1, $2 ) if $val=~/\/Info (\d+) (\d+) R/s;
     &getObj( $encryptedPdf=$1, $2 ) if $val=~/\/Encrypt (\d+) (\d+) R/s;
     $trailerObject=$val;
-    $val=$1 if $pdf=~/\D($Root 0 obj.*?endobj\s+)/s;
-    $CatalogPages=int $1 if $val=~/\/Pages (\d+) 0 R\s+/s;   
+    $val=$1 if $pdf=~/\D($Root 0 obj.*?endobj\s*)/s;#by Stefano Capuzzimato. There can be even no space after endobj (* instead of +)
+    $CatalogPages=int $1 if $val=~/\/Pages (\d+) 0 R\s*/s;
     $val="$Root 0 obj\n<<\n/Type /Catalog\n/Pages $CatalogPages 0 R\n>>\nendobj\n";# changed \r to \n for unixish systems by Alberto Accomazzi
-    $pdf=~s/(\D)$Root 0 obj.*?endobj\s+/$1$val/s; 
+    $pdf=~s/(\D)$Root 0 obj.*?endobj\s*/$1$val/s;#by Stefano Capuzzimato. There can be even no space after endobj (* instead of +)
 }
 
 sub getObj {
 	return "" if $vars{"PDFError"};
     my($obj,$instnum)=@_;
     unless ($obj[$obj] ) {
-         if ($pdf=~/\D($obj $instnum obj.*?endobj\s)/s ) {
+         if ($pdf=~/\D($obj $instnum obj.*?endobj\s*)/s ) {#by Stefano Capuzzimato. There can be even no space after endobj (* instead of +)
             $object = $1;
 #	        return "" if $object=~/\/GoToR/; # Don't want these link objects
             $obj[$obj]++;
@@ -667,10 +703,10 @@ sub uri {
 sub getPages {
 	return "" if $vars{"PDFError"};
     my($obj, $instnum)=@_;
-    my $val=$1 if $pdf=~/\s($obj $instnum obj.*?endobj\s+)/s;
+    my $val=$1 if $pdf=~/\s($obj $instnum obj.*?endobj\s*)/s;#by Stefano Capuzzimato. There can be even no space after endobj (* instead of +)
     my $found="";
     my $count=0;
-    if ($val=~/\/Kids \[\s*(.*?)\]/s ) {     
+    if ($val=~/\/Kids\s*\[\s*(.*?)\]/s ) {#by Stefano Capuzzimato. You can find spaces between "Kids" and "["
         my $kids=$1;
         $kids=~s/\s+/ /gs;
         foreach my $kid (split " R ", $kids) {      
@@ -678,7 +714,7 @@ sub getPages {
             $found.=$f;
             $count+=$c;
         }
-        $pdf=~s/(\D$obj $instnum obj.*?\/Kids \[).*?\]/$1 $found\]/s;
+        $pdf=~s/(\D$obj $instnum obj.*?\/Kids\s*\[).*?\]/$1$found\]/s;#by Stefano Capuzzimato. Between "Kids" and "[" there can be even no space
         $pdf=~s/(\D$obj $instnum obj.*?\/Count )\d+/$1$count/s;
         $found="$obj $instnum R " if $found;
     } else {
@@ -686,6 +722,7 @@ sub getPages {
 	    if ( $getPages{$pageObject{$obj}} ) {
 	        $found="$obj $instnum R ";
 	        $count=1; 
+
 	        $vars{"PDFPagesFound"}.= $vars{"PDFPagesFound"} ? ", $pageObject{$obj}" : $pageObject{$obj};
 	        $vars{"PDFPageCount"}++;
         }
@@ -738,11 +775,12 @@ Much thanks to:-
  Lyman Byrd for his welcome programming suggestions and editorial comments on the POD.
  Michael Cox for his suggestion of PDFSaveAs and for the time he spent in testing the module.
  Alberto Accomazzi for sharing his time and his knowledge of Unixish PDF voodoo magick.
-
+ Stefano Capuzzimato for correcting some stuff in the regexes he found.
 =head1 COPYRIGHT
 
 Copyright (c) 2003 by Noel Sharrock. All rights reserved.
 
+
 =head1 LICENSE
 
 This package is free software; you can redistribute it and/or modify it under the same terms as Perl itself, 
@@ -760,6 +798,7 @@ See the ``GNU General Public License'' for more details.
 
 PDF::Extract - Extracting sub PDF documents from a multipage PDF document
 
+
 =cut
 
 #------------------------------------------ End PDF Page ------------------------------------------   
